# -*- coding: iso-8859-1 -*-
"""Classe que implementa autenticacao no Downtube"""

#
# Downtube - Copyright (c) 2006 Andre Luiz <lidzky@users.sourceforge.net>
#
# $Id: DtAuth.py,v 1.9.6.1 2006/08/15 03:31:41 lidzky Exp $
#
# Este arquivo  parte do programa Downtube - http://downtube.codigolivre.org.br
#
# O Downtube  um software livre; voc pode redistrubu-lo e/ou modific-lo
# dentro dos termos da Licena Pblica Geral GNU como publicada pela
# Fundao do Software Livre (FSF); na verso 2 da Licena, ou (na sua
# opinio) qualquer verso.
#
# Este programa  distribudo na esperana que possa ser til,
# mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a
# qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a
# Licena Pblica Geral GNU para maiores detalhes.
#
# Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob
# o ttulo "LICENCA.txt", junto com este programa; se no, escreva para
# a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor,
# Boston, MA  02110-1301  US
#
#

# Python modules
from os     import system
from urllib import urlencode

# Downtube modules
from DtData import user_agent, tmpout


class Auth:
    """Authentication class for Downtube"""

    __base_login  = 'http://www.youtube.com/login?next_url='
    __base_verify = 'http://www.youtube.com/verify_age?next_url='


    def __init__(self):
        """class constructor"""

        self.__username = ''
        self.__password = ''

        self.__referer      = ''
        self.__next_url     = ''
        self.__cookies_path = ''
        self.__post_data    = ''

        self.__verbose = True
        self.__wget    = 'wget'




    def set_verbose(self, v):
        """sets wget verbosity level"""

        self.__verbose = v

        if self.__verbose:
            self.__wget = 'wget'
        else:
            self.__wget = 'wget -nv'




    def __load_login_data(self):
        """creates post data for login"""

        exception_msg = []

        if self.__next_url == '':
            exception_msg.append(messages.next_url_not_set + '\n')
        if self.__username == '':
            exception_msg.append(messages.username_not_set + '\n')
        if self.__password == '':
            exception_msg.append(messages.password_not_set + '\n')

        if len(exception_msg) > 0:
            raise AttributeError, ''.join(exception_msg)

        self.__post_data = urlencode({
                'current_form' : 'loginForm',
                'next_url'     : self.__next_url,
                'username'     : self.__username,
                'password'     : self.__password,
                'action_login' : ''
                })




    def __load_verify_data(self):
        """creates POST data for confirmation"""

        if self.__next_url == '':
            raise AttributeError, messages.next_url_not_set

        self.__post_data = urlencode({
                'next_url'       : self.__next_url,
                'action_confirm' : ''
                })




    def set_username(self, name):
        """set username for login"""

        self.__username = name




    def set_password(self, pwd):
        """set password for login"""

        self.__password = pwd




    def set_next_url(self, url):
        """set next url after login/confirm action"""

        self.__next_url = url
        self.__referer  = url




    def set_cookies_path(self, path):
        """set cookies file path"""

        self.__cookies_path = path




    def do_login(self):
        """performs login action"""

        self.__load_login_data()

        cmd = ''.join(['%s' % self.__wget,
                       ' --post-data="%s"'     % self.__post_data,
                       ' --referer="%s"'       % self.__referer,
                       ' --user-agent="%s"'    % user_agent,
                       ' --save-cookies "%s"'  % self.__cookies_path,
                       ' --keep-session-cookies',
                       ' "%s%s"'    % (self.__base_login, self.__next_url),
                       ' -O "%s"' % tmpout
                       ])

        exit = system(cmd)

        if exit != 0:
            raise OSError




    def do_verify(self):
        """performs confirm and verify action"""

        self.__load_verify_data()

        cmd = ''.join(['%s' % self.__wget,
                       ' --post-data="%s"' % self.__post_data,
                       ' --referer="%s"'       % self.__referer,
                       ' --user-agent="%s"'    % user_agent,
                       ' --load-cookies "%s"'  % self.__cookies_path,
                       ' "%s%s"'    % (self.__base_verify, self.__next_url),
                       ' -O "%s"' % tmpout
                       ])

        exit = system(cmd)

        if exit != 0:
            raise OSError

