# -*- coding: iso-8859-1 -*-
"""Classe para interpretar as paginas html baixadas pelo downtube"""

#
# Downtube - Copyright (c) 2006 Andre Luiz <lidzky@users.sourceforge.net>
#
# $Id: DtParser.py,v 1.9.6.2 2006/08/15 16:59:55 andre Exp $
#
# Este arquivo  parte do programa Downtube - http://downtube.codigolivre.org.br
#
# O Downtube  um software livre; voc pode redistrubu-lo e/ou modific-lo
# dentro dos termos da Licena Pblica Geral GNU como publicada pela
# Fundao do Software Livre (FSF); na verso 2 da Licena, ou (na sua
# opinio) qualquer verso.
#
# Este programa  distribudo na esperana que possa ser til,
# mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a
# qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a
# Licena Pblica Geral GNU para maiores detalhes.
#
# Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob
# o ttulo "LICENCA.txt", junto com este programa; se no, escreva para
# a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor,
# Boston, MA  02110-1301  US
#
#

# Downtube modules
from DtData import * 



class Parser:
    """HTML Page parser for Downtube"""


    def set_page_content(self, filepath):
        """loads page content into the object"""

        self.__page_content = (file(filepath, 'rU').readlines())




    def parse(self):
        """returns page type"""

        if self.__is_video_page():
            return VIDEO_PAGE
        elif self.__is_login_page():
            return LOGIN_PAGE
        elif self.__is_verify_page():
            return VERIFY_PAGE
        elif self.__is_failed_login():
            return LOGIN_FAILED 
        elif self.__is_private_video():
            return PRIVATE_VIDEO 
        elif self.__is_removed_video():
            return REMOVED_VIDEO 
        else:
            return UNKNOWN_PAGE




    def get_parameters(self):
        """parses video page in order to find video parameters"""

        for line in self.__page_content:
            if line.find('/player2.swf') != -1:
                start = line.find('/player2')
                end   = line.find('",', start)
                return line[start+13:end]




    def __is_video_page(self):
        """checks if it is video page or not"""

        for line in self.__page_content[5:50]:
            if line.find('<meta name="title"') != -1:
                return True

        return False




    def __is_verify_page(self):
        """checks if it is verification/confirmation page or not"""

        for line in self.__page_content:
            if line.find('<p>By clicking "Confirm') != -1:
                return True

        return False




    def __is_login_page(self):
        """checks if it is login page or not"""

        for line in self.__page_content:
            if line.find('To view this video, please verify') != -1:
                return True

        return False




    def __is_private_video(self):
        """checks if it is a private video page or not"""

        for line in self.__page_content:
            if line.find('This is a private video') != -1:
                return True

        return False




    def __is_removed_video(self):
        """checks if it is a removed video page or not"""

        for line in self.__page_content:
            if line.find('This video has been removed') != -1:
                return True

        return False




    def is_logged(self):
        """checks if user is logged in"""

        for line in self.__page_content:
            if line.find('document.logoutForm.submit') != -1:
                return True

        return False




    def __is_failed_login(self):
        """checks if it is login page or not"""

        for line in self.__page_content:
            if line.find('your login was incorrect') != -1:
                return True

        return False

