# -*- coding: iso-8859-1 -*-
"""
Classe principal do Downtube

http://downtube.codigolivre.org.br
"""

#
# Downtube - Copyright (c) 2006 Andre Luiz <lidzky@users.sourceforge.net>
#
# $Id: downtube.py,v 1.9.6.3 2006/08/16 04:56:49 lidzky Exp $
#
# Este arquivo  parte do programa Downtube - http://downtube.codigolivre.org.br
#
# O Downtube  um software livre; voc pode redistrubu-lo e/ou modific-lo
# dentro dos termos da Licena Pblica Geral GNU como publicada pela
# Fundao do Software Livre (FSF); na verso 2 da Licena, ou (na sua
# opinio) qualquer verso.
#
# Este programa  distribudo na esperana que possa ser til,
# mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a
# qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a
# Licena Pblica Geral GNU para maiores detalhes.
#
# Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob
# o ttulo "LICENCA.txt", junto com este programa; se no, escreva para
# a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor,
# Boston, MA  02110-1301  US
#

# Python modules
import sys
from os           import system, getenv, mkdir, access, remove, F_OK, R_OK
from os.path      import sep, isdir
from getopt       import getopt, GetoptError
from ConfigParser import ConfigParser, Error, NoOptionError

# downtube modules
import DtMessages as messages
from DtAuth       import Auth
from DtParser     import Parser
from DtDownloader import Downloader
from DtData       import *


class Downtube:
    """Downtube() -> Downtube object"""

    __base_url = 'http://www.youtube.com/get_video?'

    def __init__(self):
        """class constructor"""
        self.path      = ''
        self.config    = ''

        self.verbose   = False

        self.referer   = ''
        self.page_url  = ''
        self.video_id  = ''
        self.video_url = ''

        self.keep      = True
        self.convert   = False
        self.play      = False
        self.player    = ''
        self.encoder   = ''
        self.cookies   = ''
        self.username  = ''
        self.password  = ''

        self.__parms   = ''
        self.__output  = ''

        self.outflv    = ''
        self.outavi    = ''
        self.__output  = ''

        self.login      = Auth()
        self.my_parser  = Parser()
        self.downloader = Downloader()

        self.cp = ConfigParser()

        self.__using_defaults = False    




    def __error(self, msg):
        """prints msg on stderr"""

        sys.stderr.write(msg)    




    def __exit(self, code=0):
        """call to system exit with clean up action"""

        try:
            self.__cleanup()
        except:
            pass

        raise SystemExit(code)




    def __abort(self, code=1):
        """forced system exit"""

        raise SystemExit(code)




    def __cleanup(self):
        """performs clean up action on exit"""

        remove(tmpout)




    def __debug(self):
        """prints debug info"""

        print "\n*** sys ***"
        print "os          : %s" % sys.platform
        print "python      : %s" % sys.version
        print "version     : %s" % VERSION 
        print "\n*** options ***"
        print "play        : %s" % self.play
        print "keep        : %s" % self.keep
        print "player      : %s" % self.player
        print "encoder     : %s" % self.encoder
        print "convert     : %s" % self.convert
        print "username    : %s" % self.username
        print "\n*** data ***"
        print "page_url    : %s" % self.page_url
        print "video_url   : %s" % self.video_url
        print "video_id    : %s" % self.video_id




    def __print_eargs(self, error):
        """prints error args list as string"""

        try:
            for arg in error.args:
                self.__error("    %s" % arg)
            print '\n'
        except:
            pass




    def __print(self, msg):
        """print function"""

        if self.verbose:
            print msg




    def __set_verbose(self, v):
        """sets downtube verbosity level"""

        self.verbose = v
        self.downloader.set_verbose(v)
        self.login.set_verbose(v)




    def __help(self):
        """help message"""

        print messages.help




    def __usage(self):
        """usage message"""

        print "\n%s\n" % messages.usage




    def __check_command_line(self):
        """checks if command line has arguments"""

        if len(sys.argv[1:]) < 1:
            self.__usage()
            self.__exit(0)

        for a in sys.argv[1:]:
            if a == '-h':
                self.__help()
                self.__exit(0)
            elif a == '-q':
                self.__set_verbose(False)
            elif a == '-v':
                self.__set_verbose(True)
            elif a == '-V':
                print messages.welcome
                self.__exit(0)


        found_url = False 
        for a in sys.argv[1:]:
            if a.find('youtube.com') != -1 and a.find('watch?v=') != -1:
                found_url = True
                self.page_url = a
                self.referer  = a
                self.video_id = a.split('youtube.com')[1]
                break

        if not found_url:
            print "\n%s\n\n%s\n" % (messages.missing_url, messages.help_command)
            self.__exit(1)




    def __parse_command_line(self):
        """parses command line options and arguments"""

        try:
            optlist, args = getopt(sys.argv[1:], 'cde:hkno:p:qrP:U:v')
        except GetoptError, e:
            self.__error('\n%s: %s' % (messages.error, e.msg))
            self.__error('\n%s\n\n' % messages.help_command)
            self.__exit(1)
        except:
            raise

        for o, a in optlist:
            if o == "-o":
                self.__output = a
            elif o == "-e":
                self.encoder = a
                self.convert = True
            elif o == "-c":
                self.convert = True
            elif o == "-d":
                self.convert = False
                self.keep    = True
            elif o == "-k":
                self.keep = True
            elif o == "-r":
                self.play = True
            elif o == "-p":
                self.play   = True
                self.player = a
            elif o == '-U':
                self.username = a
            elif o == '-P':
                self.password = a
            elif o == '-v':
                self.__set_verbose(True)
            #elif o == '-q':
            #    self.__set_verbose(False)
            else:
                pass




    def __load_config(self):
        """reads config file"""

        if sys.platform.lower().find('win') != -1:
            # Windows (R)...
            self.__print("==> %s\n" % messages.loading_windows_config)
            self.__load_default_config()
        else:
            if self.__using_defaults == True:
                self.__load_default_config()
            else:
                try:
                    self.cookies = sep.join([self.path, 'cookies.txt'])
                    self.__load_custom_config()
                except:
                    self.__print("==> %s\n" % messages.failed_loading_custom)
                    self.__load_default_config()

        if self.__using_defaults:
            self.__print("==> %s\n" % messages.using_default_config)
        else:
            self.__print("==> %s: %s\n" % \
                         (messages.using_custom_config, self.config))




    def __load_custom_config(self):
        """loads custom config"""

        if not access(self.config, R_OK):
            self.__error("\n%s: %s\n\n" % \
                (messages.error, messages.cant_access_config))
            raise IOError
        else:
            try:
                self.__check_old_config()
            except Exception, e:
                self.__error("%s\n" % messages.cant_update_config)
                self.__print_eargs(e)
                raise

        try:
            self.cp.read(self.config)
            self.username = self.cp.get('global', 'username')
            self.password = self.cp.get('global', 'password')
            self.encoder  = self.cp.get('global', 'encoder')
            self.player   = self.cp.get('global', 'player')
            self.convert  = bool(int(self.cp.get('global', 'convert')))
            self.play     = bool(int(self.cp.get('global', 'play')))
            self.keep     = bool(int(self.cp.get('global', 'keep')))
        except:
            raise




    def __check_old_config(self):
        """provides backward compatibility with downtube 1.x config file"""

        from DtDefaults import username, password, encoder

        changed = False

        self.cp.read(self.config)

        myencoder = self.cp.get('global', 'encoder')
        if myencoder.find('INPUT') == -1 or myencoder.find('OUTPUT') == -1:
            changed = True
            self.__print("%s\n" % messages.overwriting_encoder)
            self.cp.set('global', 'encoder', encoder)

        if (not self.cp.has_option('global', 'username')):
            changed = True
            self.cp.set('global', 'username', username)
        if (not self.cp.has_option('global', 'password')):
            changed = True
            self.cp.set('global', 'password', password)

        if changed:
            self.__print("==> %s\n" % messages.updating_config_file)
            fp = file(self.config, 'w')
            self.cp.write(fp)




    def __load_default_config(self):
        """loads default configuration values"""

        # imports default config values
        from DtDefaults import play, keep, player, cookies, \
                               encoder, convert, username, password

        self.__using_defaults = True

        self.keep     = keep
        self.play     = play
        self.player   = player
        self.cookies  = cookies
        self.encoder  = encoder
        self.convert  = convert
        self.username = username
        self.password = password




    def __startup(self):
        """startup actions"""

        # ignores this method if we're on Windows(R)
        if sys.platform.lower().find('win') != -1:
            return

        dotconfig    = sep.join([getenv('HOME'), '.config'])
        self.path    = sep.join([getenv('HOME'), '.config', 'downtube'])
        self.config  = sep.join([self.path, 'downtube.conf'])
        self.cookies = sep.join([self.path, 'cookies.txt'])

        # ~/.config/
        if not isdir(dotconfig):
            try:
                mkdir(dotconfig)
            except:
                self.__error('%s: %s: %s\n' \
                    % (messages.error, messages.cant_create_dir, dotconfig))
                #TODO: raise or pass?
                #raise OSError
                self.__using_defaults = True
                return

        # ~/.config/downtube/
        if not isdir(self.path):
            try:
                mkdir(self.path)
            except:
                self.__error('%s: %s: %s\n' \
                    % (messages.error, messages.cant_create_dir, self.path))
                #TODO: raise or pass?
                self.__using_defaults = True
                return

        # ~/.config/downtube/cookies.txt
        if not access(self.cookies, F_OK):
            from shutil import copyfile
            source = sep.join([sys.prefix,'share','downtube','cookies.txt'])
            try:
                copyfile(source, self.cookies)
            except Exception, e:
                self.__print_eargs(e)
                pass

        # ~/.config/downtube/downtube.conf
        if not access(self.config, F_OK):
            from shutil import copyfile
            source = sep.join([sys.prefix,'share','downtube','downtube.conf'])
            print '==> %s %s\n' % \
                         (messages.copying_default_config, self.path)
            try:
                copyfile(source, self.config)
            except:
                # no longer raises SystemExit because now we can 
                # run downtube from sources directory ;-)
                self.__error('%s\n' % messages.cant_copy_config)
                self.__using_defaults = True
                return







    def __check_env(self):
        """performs env check"""

        # Sorry, this program is not Windows(R) friendly :-P
        if sys.platform.lower().find('win') != -1:
            return

        exit = system("which wget 2> /dev/null 1> /dev/null")
        if exit != 0:
            # oops, we can't live without wget!!!
            self.__error('\n%s: %s\n' % \
                    (messages.error, messages.wget_not_found))
            self.__exit(1)

        encoder = self.encoder.split()[0]
        player  = self.player.split()[0]

        exit = system("which %s 2> /dev/null 1> /dev/null" % encoder)
        if exit != 0:
            self.__error("%s (%s)\n" % \
                    (messages.encoder_not_found, encoder))
            self.convert = False

        exit = system("which %s 2> /dev/null 1> /dev/null" % player)
        if exit != 0:
            self.__error("\n%s (%s)\n\n" % \
                    (messages.player_not_found, player))
            self.play = False




    def __generate_filenames(self):
        """generates output filenames"""

        if self.__output == '':
            from time import time
            self.__output = 'video_' + str(time())[:10]

        self.outflv   = '"%s.flv"' % self.__output
        self.outavi   = '"%s.avi"' % self.__output




    def __start_downloads(self):
        """starts all downloads"""

        found_video_page = False
        times = 1

        self.__get_video_page()

        while not found_video_page and times <= MAX_TRIES:
            page = self.__get_page_type()
            if page == PRIVATE_VIDEO:
                if not self.my_parser.is_logged():
                    self.__do_login() 
                else:
                    self.__error(messages.invite_not_sent)
                    self.__error('\n%s: %s\n%s: %s\n\n' %\
                            (messages.username, self.username, \
                             messages.password, '<nao exibida>'))
                    self.__exit(1)
            elif page == VIDEO_PAGE:
                found_video_page = True
                break
            elif page == LOGIN_PAGE:
                self.__do_login()
            elif page == VERIFY_PAGE:
                self.__do_verify()
            elif page == LOGIN_FAILED:
                self.__error(messages.failed_login)
                self.__error('\n%s: %s\n%s: %s\n\n' %\
                        (messages.username, self.username, \
                         messages.password, self.password))
                self.__exit(1)
            elif page == REMOVED_VIDEO:
                self.__error("\n\n%s: %s\n\n" % \
                    (messages.error, messages.removed_video))
                self.__exit(0)
            else:
                self.__error(messages.unknown_page)
                self.__abort(1) 
            times += 1

        if found_video_page:
            try:
                self.__do_download()
            except OSError:
                self.__debug()
                self.__error('\n%s: %s\n\n' % \
                        (messages.error, messages.cant_download_video))
                self.__abort(1)
            except KeyboardInterrupt:
                self.__print("\n\n%s\n" % messages.exiting_on_cancel)
                self.__exit(0)
            except SystemExit:
                raise
            except Exception, e:
                self.__debug()
                self.__print_eargs(e)
                self.__exit(1)

        elif times > MAX_TRIES:
            self.__error("\n%s: %s\n\n" % \
                            (messages.error, messages.max_tries_reached))
            self.__abort(1)




    def __get_video_page(self):
        """downloads video page"""

        self.__print("==> %s\n" % messages.downloading_video_page)

        try:
            self.downloader.get_video_page(self.page_url, self.cookies)
        except KeyboardInterrupt:
            self.__print("\n\n%s\n" % messages.exiting_on_cancel)
            self.__exit(0)
        except SystemExit:
            raise
        except Exception, e:
            self.__print_eargs(e)
            self.__error('%s: %s\n\n' \
                    % (messages.error, messages.cant_download_video_page))
            self.__abort(1)




    def __get_page_type(self):
        """parses downloaded page in order to figure out which action to take"""

        self.__print('==> %s\n' % messages.parsing_page)

        try:
            self.my_parser.set_page_content(tmpout)
        except IOError:
            self.__debug()
            self.__error("%s: %s '%s'" \
                    % (messages.error, messages.cant_read, tmpout))
            self.__exit(1)

        return self.my_parser.parse()




    def __do_download(self):
        """video download"""

        self.__parms = self.__get_parameters()

        try:
            self.__get_video()
        except:
            raise




    def __do_login(self):
        """performs login action"""

        self.__print("==> %s\n" % messages.video_requires_auth)

        self.login.set_username(self.username)
        self.login.set_password(self.password)
        self.login.set_next_url(self.video_id)
        self.login.set_cookies_path(self.cookies)

        try:
            self.login.do_login()
        except Exception, e:
            self.__print_eargs(e)
            self.__exit(1)




    def __do_verify(self):
        """performs confirm/verify action"""

        self.__print("==> %s\n" % messages.video_requires_verify)

        self.login.set_next_url(self.video_id)
        self.login.set_cookies_path(self.cookies)

        try:
            self.login.do_verify()
        except Exception, e:
            self.__error("\n%s: %s\n\n" % \
                    (messages.error, messages.verification_failed))
            self.__print_eargs(e)
            self.__exit(1)




    def __get_parameters(self):
        """get video page parameters"""

        return self.my_parser.get_parameters()




    def __get_video(self):
        """downloads video file"""

        self.__print("==> %s\n" % messages.downloading_video)

        self.video_url = "%s%s" % (self.__base_url, self.__parms)
        try:
            self.downloader.get_video(self.video_url, self.referer, \
                                      self.outflv, self.cookies)
        except KeyboardInterrupt:
            print "\n\n%s\n\n" % messages.exiting_on_cancel
            self.__exit(0)
        except SystemExit:
                raise
        except Exception, e:
            self.__print_eargs(e)
            self.__error('\n%s: %s\n\n' \
                    % (messages.error, messages.cant_download_video))
            self.__abort(1)

        print "\n\n==> %s %s\n" % (messages.file_saved_as, self.outflv)




    def __handle_videos(self):
        """handles video files"""

        if self.convert == True:
            try:
                self.__convert_video()
            except Exception, e:
                self.convert = False
                self.__print_eargs(e)
                self.__error('\n%s: %s\n\n' \
                        % (messages.error, messages.cant_convert_video))

        if self.play == True:
            try:
                self.__play_video()
            except:
                self.__error('\n%s: %s\n\n' \
                        % (messages.error, messages.cant_play_video))




    def __convert_video(self):
        """converts video from flv to another format (defaults to avi)"""
        
        self.encoder = self.encoder.replace('OUTPUT', self.outavi)
        self.encoder = self.encoder.replace('INPUT', self.outflv)
        
        cmd = "%s" % self.encoder
        self.__print("==> %s:\n    %s\n" % (messages.converting_video, cmd))
        exit = system(cmd)

        if exit != 0:
            # this will force __play_video to play the .FLV instead of .AVI
            self.convert = False
            raise OSError

        print "\n\n==> %s %s\n" % (messages.file_saved_as, self.outavi)




    def __play_video(self):
        """plays the video"""

        if self.convert == True:
            cmd = "%s %s" % (self.player, self.outavi)
            self.__print("==> %s:\n    %s\n\n" % (messages.playing_video, cmd))
            exit = system(cmd)
            if exit != 0: 
                raise Exception
        elif self.keep == True:
            cmd = "%s %s" % (self.player, self.outflv)
            self.__print("==> %s:\n    %s\n" % (messages.playing_video, cmd))
            exit = system(cmd)
            if exit != 0:
                raise Exception
        else:
            self.__error(messages.no_video_to_play)




    def __end_main(self):
        """end of main method"""
        try:
            if self.keep == False:
                self.__print("%s %s" % (messages.deleting, self.outflv))
                remove(self.outflv)
    
            self.__cleanup()
        except:
            pass
        
        print "\n\n%s\n\n" % messages.goodbye




    def __welcome(self):
        """prints welcome message"""

        print '\n'
        print '*' * 60
        print " %s" % messages.welcome
        print " %s" % SITE
        print '*' * 60
        print '\n'




    def main(self):
        """main downtube method"""

        # let the magic begin!
        self.__check_command_line()
        self.__welcome()
        self.__startup()
        self.__load_config()
        self.__parse_command_line()
        self.__check_env()
        self.__generate_filenames()
        self.__start_downloads()
        self.__handle_videos()
        self.__end_main()



if __name__ == '__main__':
    sys.stderr.write('%s\n' % messages.dont_execute_downtubepy)
    raise SystemExit(1)
